function [ output_args ] = assignTopics( ALPHA, BETA, T, NITER)
%
%   Author: Shaohua Wan.   
%
%   Implementation based on:
%       http://psiexp.ss.uci.edu/research/programs_data/exampleimages1.html

addpath('topictoolbox');

image_dir = 'images';

% ALPHA = 1;   % ALPHA hyperparameter
% BETA  = 1;   % BETA hyperparameter
% T = 30;
% NITER = 100;
seed  = 4; % seed for the random number generator

load('data\visualwords_200.mat');  % load variable T_all from file.
D = length(T_all);
ns = length(T_all(1).x);
w = zeros(D*ns,1);
d = zeros(size(w));

words_topics_dir = 'wds_tpcs';
words_topics_dir = sprintf('%s%05.2f_%05.2f_%02d_%03d_%03d_%05d',words_topics_dir, ALPHA, BETA, T, NITER, D, ns);
if(~isdir(words_topics_dir))
    mkdir(words_topics_dir);
end

for j=1:D
    w( 1+ns*(j-1) : ns*j ) = T_all(j).data;
    d( 1+ns*(j-1) : ns*j ) = j*ones(ns,1);
end

N=NITER;
[ WP,DP,Z ] = GibbsSamplerLDA( w , d , T , N , ALPHA , BETA , seed , 0 );
visualizeTopics(T_all, Z, image_dir, words_topics_dir, T);

str = sprintf('%s\t=\t%f \r\n %s\t=\t%f \r\n %s\t=\t%f \r\n %s\t=\t%f \r\n %s\t=\t%f \r\n %s\t=\t%f',...
    'ALPHA', ALPHA, 'BETA', BETA, 'T', T, 'NITER', NITER, 'D', D,...
    'ns', ns);

fileID = fopen([words_topics_dir '\readme.txt'],'w');
fprintf(fileID, str);
fclose(fileID);

end


function visualizeTopics(T_all, Z, image_dir, words_topics_dir, T)
%
%   Visualize topics on the images
%
%   Assign different color to the visual words of the images.

dnsample = 1;

D = length(T_all);
ns = length(T_all(1).x);
w = zeros(D*ns,1);
d = zeros(size(w));

%hsv_colormp = colormap(hsv(T));
hsv_colormp = colormap(hsv(T+1));
hsv_colormp = hsv_colormp*255;

for i=1:D
    img_name = T_all(i).filename;
    img_w = T_all(i).wid;
    img_h = T_all(i).hgt;
    x_crds = T_all(i).x;
    y_crds = T_all(i).y;
    tp_inds = Z(1+(i-1)*ns: i*ns);
    img = imread(fullfile(image_dir, img_name));
    img = setImgPixelsVaules(img, x_crds, y_crds, tp_inds, hsv_colormp, dnsample);
    out_img_name = fullfile(words_topics_dir, img_name);
    imwrite(img, out_img_name);
end

end

function img = setImgPixelsVaules(img, xs, ys, vals, colormp, dnsample)

% img should be a color image

xs_uniq = unique(xs);
ys_uniq = unique(ys);

xs_dnsample = downsample(xs_uniq, dnsample);
ys_dnsample = downsample(ys_uniq, dnsample);

assert(length(size(img)) == 3); % color image
img = rgb2gray(img);

for i=1:length(xs)
    idx1 = find(xs_dnsample==xs(i), 1);
    idx2 = find(ys_dnsample==ys(i), 1);
    if( isempty(idx1) || isempty(idx2) )
        continue;
    else
        img = insertShape(img,'FilledCircle',[xs(i) ys(i) 3],...
            'Color', colormp(vals(i),:));
    end
end


end