function [ output_args ] = img2Words(  imageFileList, imageBaseDir, dataBaseDir, params, canSkip, saveSift )
%
%   Author: Shaohua Wan
%       
%   Process images, extract visual words, and save them in disk.
%
%   Input:
%       For now, just ignore all input argument.
%       Set these arguments inside this function.
%
%   Output:
%       visualwords_200.mat, <n_imgs x 1 struct> structure type,
%       with the following structure
%              T_all
%                  data: visual word indices
%                  x: x-coordinates of the visual words
%                  y: y-coordinates of the visual words
%                  wid: width of the image
%                  hgt: height of the image
%                  filename: the file name of the image

addpath('SpatialPyramid');
addpath('topictoolbox');

image_dir = 'images'; 
data_dir = 'data';

fnames = dir(fullfile(image_dir, '*.jpg'));
num_files = size(fnames,1);
filenames = cell(num_files,1);

for f = 1:num_files
	filenames{f} = fnames(f).name;
end

%% parameters for feature extraction (see GenerateSiftDescriptors)

if(~exist('params','var'))
    params.maxImageSize = 1000
    params.gridSpacing = 8
    params.patchSize = 16
    params.dictionarySize = 200
    params.numTextonImages = 50
    params.pyramidLevels = 3
    params.oldSift = false;
end


if(~isfield(params,'maxImageSize'))
    params.maxImageSize = 1000
end
if(~isfield(params,'gridSpacing'))
    params.gridSpacing = 8
end
if(~isfield(params,'patchSize'))
    params.patchSize = 16
end
if(~isfield(params,'dictionarySize'))
    params.dictionarySize = 200
end
if(~isfield(params,'numTextonImages'))
    params.numTextonImages = 400
end
if(~isfield(params,'pyramidLevels'))
    params.pyramidLevels = 3
end
if(~isfield(params,'oldSift'))
    params.oldSift = false
end

if(~exist('canSkip','var'))
    canSkip = 1
end
if(~exist('saveSift','var'))
    saveSift = 1
end

imageFileList = filenames;
imageBaseDir = image_dir;
dataBaseDir = data_dir;

pfig = sp_progress_bar('Building Spatial Pyramid');
%% build the pyramid
if(saveSift)
    GenerateSiftDescriptors( imageFileList,imageBaseDir,dataBaseDir,params,canSkip,pfig);
end
CalculateDictionary(imageFileList,imageBaseDir,dataBaseDir,'_sift.mat',params,canSkip,pfig);
canSkip = 0;
cvtImgs2words(imageFileList,imageBaseDir,dataBaseDir,'_sift.mat',params,canSkip,pfig);


end

